'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const $listeners = Symbol("EventEmitter.listeners");
class EventEmitter {
  constructor() {
    Object.defineProperty(this, $listeners, {
      value: Object.create(null)
    });
  }
  on(arg, fn) {
    if (arguments.length == 2) {
      if (fn)
        this._addListener(arg, fn);
      return fn;
    }
    for (let key in arg) {
      if (fn = arg[key])
        this._addListener(key, fn);
    }
    return this;
  }
  off(key, fn) {
    if (arguments.length == 2) {
      if (fn)
        this._removeListener(key, fn);
    } else if (key == "*") {
      Object.keys(this[$listeners]).forEach((key2) => this.off(key2));
    } else {
      this.getListeners(key).forEach((fn2) => this._removeListener(key, fn2));
    }
    return this;
  }
  emit(key, ...args) {
    if (getListeners(this, "emit")) {
      this._emit("emit", [key, args]);
    }
    this._emit(key, args);
  }
  hasListeners(key) {
    return !!getListeners(this, key);
  }
  getListeners(key) {
    return arguments.length ? getListeners(this, key) || emptySet : this[$listeners];
  }
  _addListeners(key) {
    return this[$listeners][key] = new Set();
  }
  _removeListeners(key) {
    delete this[$listeners][key];
  }
  _addListener(key, fn) {
    let list = getListeners(this, key) || this._addListeners(key);
    list.add(fn);
  }
  _removeListener(key, fn) {
    let list = getListeners(this, key);
    if (list && list.delete(fn)) {
      if (!list.size) {
        this._removeListeners(key);
      }
      return true;
    }
  }
  _emit(key, args) {
    let list = getListeners(this, key);
    if (list) {
      list.forEach((fn) => fn(...args) !== false || this._removeListener(key, fn));
      if (!list.size) {
        this._removeListeners(key);
      }
    }
  }
}
const emptySet = new Set();
const getListeners = (ee, key) => ee[$listeners][key];

exports.EventEmitter = EventEmitter;
//# sourceMappingURL=ee.js.map
